# serialization keys
OVEN_PORT = "oven_port"
SETPOINT = "setpoint"
INCREMENT = "increment"
MEASUREMENT_INTERVAL = "measurement_interval"
MINIMUM_MEASUREMENTS = "minimum_measurements"
TOLERANCE = "tolerance"


# files
TEMPERATURES_FILENAME = "temperatures.csv"

# labels
OVEN_PORT_LABEL = "Oven Port"
SETPOINT_LABEL = "Setpoint"
INCREMENT_LABEL = "Increment"
INTERVAL_LABEL = "Measurement Interval"
MINIMUM_MEASUREMENTS_LABEL = "Minimum Measurements for Stability"
TOLERANCE_LABEL = "Stability Tolerance"
